<?php
/**
 * 日志相关
 * @作者 qinggan <admin@phpok.com>
 * @主页 http://www.phpok.com
 * @版本 4.x
 * @许可 http://www.phpok.com/lgpl.html PHPOK开源授权协议：GNU Lesser General Public License
 * @时间 2021年5月31日
**/
if(!defined("PHPOK_SET")){exit("<h1>Access Denied</h1>");}
class log_model_base extends phpok_model
{
	public function __construct()
	{
		parent::model();
	}

	/**
	 * 保存日志
	 * @参数 $note 日志说明
	 * @参数 $mask 是否手动标记，为true时表示手动标志
	**/
	public function save($note='',$mask=false)
	{
		if(!$note){
			$tmpfile = $this->app_id.'/'.$this->ctrl.'_control.php';
			$note = P_Lang('执行文件{ctrl}方法：{func}',array('ctrl'=>$tmpfile,'func'=>$this->func.'_f'));
		}
		//传过来的日志说明为数组或对像都是手动标志
		if(is_array($note) || is_object($note)){
			$note = print_r($note,true);
			$mask = true;
		}
		if($note && is_string($note)){
			$note = strip_tags($note);
			$note = str_replace(array("'",'"',"\t","\n","\r","&nbsp;"," "),'',$note);
			$note = phpok_cut($note,255,'…');
		}
		$note = addslashes(trim($note));
		$ip = $this->lib('common')->ip();
		$data = array('note'=>$note,'dateline'=>$this->time,'app_id'=>$this->app_id,'ip'=>$ip);
		if($this->session->val('admin_id') && $this->app_id == 'admin'){
			$data['admin_id'] = $this->session->val('admin_id');
			$data['account'] = $this->session->val('admin_account');
		}else{
			$data['admin_id'] = 0;
		}
		if($this->session->val('user_id') && $this->app_id != 'admin'){
			$data['user_id'] = $this->session->val('user_id');
			$data['user'] = $this->session->val('user_name');
		}else{
			$data['user_id'] = 0;
		}
		$data['ctrl'] = $this->ctrl;
		$data['func'] = $this->func;
		$data['appid'] = $this->app_id;
		$data['mask'] = $mask ? 1 : 0;
		$url = $this->lib('server')->https() ? 'https://' : 'http://';
		$url.= $this->lib('server')->domain($this->config['get_domain_method']);
		$port = $this->lib('server')->port();
		if($port != 80 && $port != 443){
			$url .= ':'.$port;
		}
		$url .= $this->lib('server')->uri();
		$referer = $this->lib('server')->referer();
		$data['url'] = $this->format($url);
		$data['referer'] = $this->format($referer);
		$data['session_id'] = $this->session->sessid();
		return $this->_save($data);
	}

	private function _save($data)
	{
		if(!$data){
			return false;
		}
		ksort($data);
		$fields = array_keys($data);
		$tmp = implode(",",$fields).'[:[:]:]'.implode('[:[:]:]',$data);
		$file = $this->dir_data."log/".date("Ymd",$this->time).'.php';
		if(!file_exists($file)){
			file_put_contents($file,"<?php exit();?>\n");
		}
		$handle = fopen($file,'ab');
		fwrite($handle,$tmp."\n");
		fclose($handle);
	}

	/**
	 * 取得日志列表
	 * @参数 $condition 查询条件
	 * @参数 $offset 开始位置，首位从0计起
	 * @参数 $psize 每次读取数量
	**/
	public function get_list($condition='')
	{
		$file = $this->dir_data.'log/'.$condition.'.php';
		if(!file_exists($file)){
			return false;
		}
		$list = file($file);
		unset($list[0]);
		if(!$list || count($list)<1){
			return false;
		}
		$rslist = array();
		foreach($list as $key=>$value){
			$tmp = $this->string2array($value);
			$rslist[] = $tmp;
		}
		krsort($rslist);
		return $rslist;
	}

	private function string2array($string)
	{
		$tmp = explode("[:[:]:]",$string);
		$code = explode(",",$tmp[0]);
		$rs = array();
		foreach($tmp as $key=>$value){
			if($key <1){
				continue;
			}
			$id = $code[($key-1)];
			$rs[$id] = $value;
		}
		return $rs;
	}

	/**
	 * 取得日志数量
	 * @参数 $condition 查询条件
	**/
	public function get_count($condition='')
	{
		$sql  = "SELECT count(l.id) FROM ".$this->db->prefix."log l ";
		$sql .= "LEFT JOIN ".$this->db->prefix."adm a ON(l.admin_id=a.id) ";
		$sql .= "LEFT JOIN ".$this->db->prefix."user u ON(l.user_id=u.id) ";
		if($condition){
			$sql.= "WHERE ".$condition." ";
		}
		return $this->db->count($sql);
	}

}
